@@ -24,6 +24,7 @@ import tensorflow_datasets as tds
 
 from ..configs.config import DecoderConfig
 from ..utils.utils import preprocess_paths
+from . import wordpiece
 
 ENGLISH_CHARACTERS = [" ", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m",
                       "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "'"]
@@ -37,6 +38,21 @@ class TextFeaturizer(metaclass=abc.ABCMeta):
         self.tokens2indices = {}
         self.tokens = []
         self.num_classes = None
+        self.max_length = 0
+
+    @property
+    def shape(self) -> list:
+        return [self.max_length if self.max_length > 0 else None]
+
+    @property
+    def prepand_shape(self) -> list:
+        return [self.max_length + 1 if self.max_length > 0 else None]
+
+    def update_length(self, length: int):
+        self.max_length = max(self.max_length, length)
+
+    def reset_length(self):
+        self.max_length = 0
 
     def preprocess_text(self, text):
         text = unicodedata.normalize("NFC", text.lower())
@@ -307,6 +323,114 @@ class SubwordFeaturizer(TextFeaturizer):
             return tf.gather_nd(upoints, tf.where(tf.not_equal(upoints, 0)))
 
 
+class TFSubwordFeaturizer(TextFeaturizer):
+    """
+    Extract text feature based on char-level granularity.
+    By looking up the vocabulary table, each line of transcript will be
+    converted to a sequence of integer indexes.
+    """
+
+    def __init__(self, decoder_config: dict, subwords=None):
+        """
+        decoder_config = {
+            "target_vocab_size": int,
+            "max_subword_length": 4,
+            "max_corpus_chars": None,
+            "reserved_tokens": None,
+            "beam_width": int,
+            "lm_config": {
+                ...
+            }
+        }
+        """
+        super(TFSubwordFeaturizer, self).__init__(decoder_config)
+        self.subwords = self.__load_subwords() if subwords is None else subwords
+        self.blank = 0  # subword treats blank as 0
+        self.num_classes = self.subwords.vocab_size
+
+    def __load_subwords(self):
+        return wordpiece.WordpieceTokenizer(self.decoder_config.vocabulary, token_out_type=tf.int32)
+
+    @classmethod
+    def build_from_corpus(cls, decoder_config: dict, corpus_files: list = None, output_file: str = None):
+        dconf = DecoderConfig(decoder_config.copy())
+        corpus_files = dconf.corpus_files if corpus_files is None or len(corpus_files) == 0 else corpus_files
+        filename = dconf.vocabulary if output_file is None else preprocess_paths(output_file)
+
+        def corpus_generator():
+            for file in corpus_files:
+                with open(file, "r", encoding="utf-8") as f:
+                    lines = f.read().splitlines()
+                    lines = lines[1:]
+                for line in lines:
+                    line = line.split("\t")
+                    yield line[-1]
+
+        wordpiece.build_from_corpus(
+            corpus_generator(),
+            output_file_path=filename,
+            target_vocab_size=dconf.target_vocab_size,
+            max_subword_length=dconf.max_subword_length,
+            max_corpus_chars=dconf.max_corpus_chars,
+            reserved_tokens=dconf.reserved_tokens
+        )
+
+        subwords = wordpiece.WordpieceTokenizer(filename, token_out_type=tf.int32)
+        return cls(decoder_config, subwords)
+
+    @classmethod
+    def load_from_file(cls, decoder_config: dict, filename: str = None):
+        dconf = DecoderConfig(decoder_config.copy())
+        filename = dconf.vocabulary if filename is None else preprocess_paths(filename)
+        subwords = wordpiece.WordpieceTokenizer(filename, token_out_type=tf.int32)
+        return cls(decoder_config, subwords)
+
+    def extract(self, text: tf.Tensor) -> tf.Tensor:
+        """
+        Convert string to a list of integers
+        Args:
+            text: string (sequence of characters)
+
+        Returns:
+            sequence of ints in tf.Tensor
+        """
+        indices = self.subwords.tokenize(text)
+        indices = indices.merge_dims(0, -1)
+        return indices.to_tensor()
+
+    def iextract(self, indices: tf.Tensor) -> tf.Tensor:
+        """
+        Convert list of indices to string
+        Args:
+            indices: tf.Tensor with dim [B, None]
+
+        Returns:
+            transcripts: tf.Tensor of dtype tf.string with dim [B]
+        """
+        with tf.device("/CPU:0"):  # string data is not supported on GPU
+            indices = self.normalize_indices(indices)
+            text = self.subwords.detokenize(indices)
+            return tf.strings.reduce_join(text, separator=" ", axis=-1)
+
+    @tf.function(
+        input_signature=[
+            tf.TensorSpec([None], dtype=tf.int32)
+        ]
+    )
+    def indices2upoints(self, indices: tf.Tensor) -> tf.Tensor:
+        """
+        Transform Predicted Indices to Unicode Code Points (for using tflite)
+        Args:
+            indices: tf.Tensor of Classes in shape [None]
+
+        Returns:
+            unicode code points transcript with dtype tf.int32 and shape [None]
+        """
+        with tf.name_scope("indices2upoints"):
+            text = self.iextract(tf.expand_dims(indices, axis=0))
+            return tf.reshape(text, shape=[-1])
+
+
 class SentencePieceFeaturizer(TextFeaturizer):
     """
     Extract text feature based on sentence piece package.
@@ -317,7 +441,7 @@ class SentencePieceFeaturizer(TextFeaturizer):
     PAD_TOKEN, PAD_TOKEN_ID = "<pad>", 0  # unused, by default
 
     def __init__(self, decoder_config: dict, model=None):
-        super().__init__(decoder_config)
+        super(SentencePieceFeaturizer, self).__init__(decoder_config)
         self.model = model
         self.blank = 0  # treats blank as 0 (pad)
         self.upoints = None
@@ -335,9 +459,9 @@ class SentencePieceFeaturizer(TextFeaturizer):
     @classmethod
     def build_from_corpus(cls, decoder_config: dict):
         """
-        --model_prefix: output model name prefix. <model_name>.model and <model_name>.vocab are generated.  
+        --model_prefix: output model name prefix. <model_name>.model and <model_name>.vocab are generated.
         --vocab_size: vocabulary size, e.g., 8000, 16000, or 32000
-        --model_type: model type. Choose from unigram (default), bpe, char, or word. 
+        --model_type: model type. Choose from unigram (default), bpe, char, or word.
         The input sentence must be pretokenized when using word type."""
         decoder_cfg = DecoderConfig(decoder_config)
         # Train SentencePiece Model
